@@ -24,7 +24,7 @@ import (
 	"os"
 	"testing"
 
-	"github.com/coreos/rkt/pkg/selinux"
+	"github.com/rkt/rkt/pkg/selinux"
 )
 
 func TestSetfilecon(t *testing.T) {
