@@ -17,14 +17,14 @@ package flag
 import (
 	"strings"
 
-	pkgflag "github.com/coreos/rkt/pkg/flag"
+	pkgflag "github.com/rkt/rkt/pkg/flag"
 )
 
 const (
 	insecureNone  = 0
 	insecureImage = 1 << (iota - 1)
 	insecureTLS
-	insecureOnDisk
+	insecureOnDisk // ignored, left for backwards compatibility
 	insecureHTTP
 	insecurePubKey
 	insecureCapabilities
@@ -32,7 +32,7 @@ const (
 	insecureSeccomp
 
 	insecureAllFetch = (insecureImage | insecureTLS | insecureHTTP | insecurePubKey)
-	insecureAllRun   = (insecureOnDisk | insecureCapabilities | insecurePaths | insecureSeccomp)
+	insecureAllRun   = (insecureCapabilities | insecurePaths | insecureSeccomp)
 	insecureAll      = (insecureAllFetch | insecureAllRun)
 )
 
@@ -85,6 +85,8 @@ func NewSecFlags(defOpts string) (*SecFlags, error) {
 	if err != nil {
 		return nil, err
 	}
+	// Ignore ondisk
+	bf.Flags = bf.Flags &^ insecureOnDisk
 
 	sf := &SecFlags{
 		BitFlags: bf,
@@ -100,10 +102,6 @@ func (sf *SecFlags) SkipTLSCheck() bool {
 	return sf.HasFlag(insecureTLS)
 }
 
-func (sf *SecFlags) SkipOnDiskCheck() bool {
-	return sf.HasFlag(insecureOnDisk)
-}
-
 func (sf *SecFlags) AllowHTTP() bool {
 	return sf.HasFlag(insecureHTTP)
 }
