@@ -26,16 +26,16 @@ import (
 
 	"github.com/appc/spec/schema/types"
 	cnitypes "github.com/containernetworking/cni/pkg/types"
-	"github.com/coreos/rkt/common"
-	"github.com/coreos/rkt/pkg/label"
-	"github.com/coreos/rkt/pkg/lock"
-	pkgPod "github.com/coreos/rkt/pkg/pod"
-	"github.com/coreos/rkt/pkg/user"
-	"github.com/coreos/rkt/rkt/image"
-	"github.com/coreos/rkt/stage0"
-	"github.com/coreos/rkt/store/imagestore"
-	"github.com/coreos/rkt/store/treestore"
 	"github.com/hashicorp/errwrap"
+	"github.com/rkt/rkt/common"
+	"github.com/rkt/rkt/pkg/label"
+	"github.com/rkt/rkt/pkg/lock"
+	pkgPod "github.com/rkt/rkt/pkg/pod"
+	"github.com/rkt/rkt/pkg/user"
+	"github.com/rkt/rkt/rkt/image"
+	"github.com/rkt/rkt/stage0"
+	"github.com/rkt/rkt/store/imagestore"
+	"github.com/rkt/rkt/store/treestore"
 	"github.com/spf13/cobra"
 )
 
@@ -77,6 +77,8 @@ image arguments with a lone "---" to resume argument parsing.`,
 	flagUUIDFileSave string
 	flagHostname     string
 	flagHostsEntries flagStringList
+	flagPullPolicy   string
+	flagIPCMode      string
 )
 
 func addIsolatorFlags(cmd *cobra.Command, compat bool) {
@@ -106,9 +108,18 @@ func addAppFlags(cmd *cobra.Command) {
 	cmd.Flags().Var((*appGroup)(&rktApps), "group", "group override for the preceding image (example: '--group=group')")
 	cmd.Flags().Var((*appSupplementaryGIDs)(&rktApps), "supplementary-gids", "supplementary group IDs override for the preceding image (examples: '--supplementary-gids=1024,2048'")
 	cmd.Flags().Var((*appName)(&rktApps), "name", "set the name of the app (example: '--name=foo'). If not set, then the app name default to the image's name")
-	cmd.Flags().Var((*appAnnotation)(&rktApps), "user-annotation", "set the app's annotations (example: '--user-annotation=foo=bar')")
+	cmd.Flags().Var((*appAnnotation)(&rktApps), "annotation", "set the app's annotations (example: '--annotation=foo=bar')")
+	cmd.Flags().Var((*appUserAnnotation)(&rktApps), "user-annotation", "set the app's annotations (example: '--user-annotation=foo=bar')")
 	cmd.Flags().Var((*appLabel)(&rktApps), "user-label", "set the app's labels (example: '--user-label=foo=bar')")
 	cmd.Flags().Var((*appEnv)(&rktApps), "environment", "set the app's environment variables (example: '--environment=foo=bar')")
+	if common.IsExperimentEnabled("attach") {
+		cmd.Flags().Var((*appStdin)(&rktApps), "stdin", "stdin mode for the preceding application (example: '--stdin=null')")
+		cmd.Flags().MarkHidden("stdin")
+		cmd.Flags().Var((*appStdout)(&rktApps), "stdout", "stdout mode for the preceding application (example: '--stdout=log')")
+		cmd.Flags().MarkHidden("stdout")
+		cmd.Flags().Var((*appStderr)(&rktApps), "stderr", "stderr mode for the preceding application (example: '--stderr=log')")
+		cmd.Flags().MarkHidden("stderr")
+	}
 }
 
 func init() {
@@ -135,12 +146,16 @@ func init() {
 	cmdRun.Flags().StringVar(&flagDNSDomain, "dns-domain", "", "DNS domain to write in /etc/resolv.conf")
 	cmdRun.Flags().Var(&flagHostsEntries, "hosts-entry", "Entries to add to the pod-wide /etc/hosts. Pass 'host' to use the host's /etc/hosts")
 	cmdRun.Flags().BoolVar(&flagStoreOnly, "store-only", false, "use only available images in the store (do not discover or download from remote URLs)")
+	cmdRun.Flags().MarkDeprecated("store-only", "please use --pull-policy=never")
 	cmdRun.Flags().BoolVar(&flagNoStore, "no-store", false, "fetch images ignoring the local store")
+	cmdRun.Flags().MarkDeprecated("no-store", "please use --pull-policy=update")
+	cmdRun.Flags().StringVar(&flagPullPolicy, "pull-policy", image.PullPolicyNew, "when to pull an image")
 	cmdRun.Flags().StringVar(&flagPodManifest, "pod-manifest", "", "the path to the pod manifest. If it's non-empty, then only '--net', '--no-overlay' and '--interactive' will have effect")
 	cmdRun.Flags().BoolVar(&flagMDSRegister, "mds-register", false, "register pod with metadata service. needs network connectivity to the host (--net=(default|default-restricted|host)")
 	cmdRun.Flags().StringVar(&flagUUIDFileSave, "uuid-file-save", "", "write out pod UUID to specified file")
 	cmdRun.Flags().StringVar(&flagHostname, "hostname", "", `pod's hostname. If empty, it will be "rkt-$PODUUID"`)
 	cmdRun.Flags().Var((*appsVolume)(&rktApps), "volume", "volumes to make available in the pod")
+	cmdRun.Flags().StringVar(&flagIPCMode, "ipc", "", `whether to stay in the host IPC namespace. Syntax: --ipc=[auto|private|parent]`)
 
 	// per-app flags
 	cmdRun.Flags().Var((*appAsc)(&rktApps), "signature", "local signature file to use in validating the preceding image")
@@ -171,6 +186,12 @@ func runRun(cmd *cobra.Command, args []string) (exit int) {
 		stderr.Print("both --store-only and --no-store specified")
 		return 254
 	}
+	if flagStoreOnly {
+		flagPullPolicy = image.PullPolicyNever
+	}
+	if flagNoStore {
+		flagPullPolicy = image.PullPolicyUpdate
+	}
 
 	if flagPrivateUsers {
 		if !common.SupportsUserNS() {
@@ -196,8 +217,9 @@ func runRun(cmd *cobra.Command, args []string) (exit int) {
 
 	if len(flagPodManifest) > 0 && (rktApps.Count() > 0 ||
 		(*appsVolume)(&rktApps).String() != "" || (*appMount)(&rktApps).String() != "" ||
-		len(flagPorts) > 0 || flagStoreOnly || flagNoStore ||
-		flagInheritEnv || !flagExplicitEnv.IsEmpty() || !flagEnvFromFile.IsEmpty()) {
+		len(flagPorts) > 0 || flagPullPolicy == image.PullPolicyNever ||
+		flagPullPolicy == image.PullPolicyUpdate || flagInheritEnv ||
+		!flagExplicitEnv.IsEmpty() || !flagEnvFromFile.IsEmpty()) {
 		stderr.Print("conflicting flags set with --pod-manifest (see --help)")
 		return 254
 	}
@@ -246,9 +268,8 @@ func runRun(cmd *cobra.Command, args []string) (exit int) {
 		Debug:              globalFlags.Debug,
 		TrustKeysFromHTTPS: globalFlags.TrustKeysFromHTTPS,
 
-		StoreOnly: flagStoreOnly,
-		NoStore:   flagNoStore,
-		WithDeps:  true,
+		PullPolicy: flagPullPolicy,
+		WithDeps:   true,
 	}
 	if err := fn.FindImages(&rktApps); err != nil {
 		stderr.Error(err)
@@ -275,7 +296,6 @@ func runRun(cmd *cobra.Command, args []string) (exit int) {
 		stderr.PrintE("error initialising SELinux", err)
 		return 254
 	}
-
 	p.MountLabel = mountLabel
 
 	cfg := stage0.CommonConfig{
@@ -304,10 +324,9 @@ func runRun(cmd *cobra.Command, args []string) (exit int) {
 	useOverlay := !flagNoOverlay && ovlOk
 
 	pcfg := stage0.PrepareConfig{
-		CommonConfig:       &cfg,
-		UseOverlay:         useOverlay,
-		PrivateUsers:       privateUsers,
-		SkipTreeStoreCheck: globalFlags.InsecureFlags.SkipOnDiskCheck(),
+		CommonConfig: &cfg,
+		UseOverlay:   useOverlay,
+		PrivateUsers: privateUsers,
 	}
 
 	if len(flagPodManifest) > 0 {
@@ -378,6 +397,7 @@ func runRun(cmd *cobra.Command, args []string) (exit int) {
 		InsecureSeccomp:      globalFlags.InsecureFlags.SkipSeccomp(),
 		UseOverlay:           useOverlay,
 		HostsEntries:         *HostsEntries,
+		IPCMode:              flagIPCMode,
 	}
 
 	_, manifest, err := p.PodManifest()
@@ -537,10 +557,10 @@ func (e *envFileMap) Set(s string) error {
 		pair := strings.SplitN(line, "=", 2)
 		// Malformed lines
 		if len(pair) != 2 || len(pair[0]) == 0 {
-			return fmt.Errorf("environment variable must be specified as name=value (file %q)", file)
+			return fmt.Errorf("environment variable must be specified as name=value (file %q)", s)
 		}
 		if _, exists := e.mapping[pair[0]]; exists {
-			return fmt.Errorf("environment variable %q already set (file %q)", pair[0], file)
+			return fmt.Errorf("environment variable %q already set (file %q)", pair[0], s)
 		}
 		e.mapping[pair[0]] = pair[1]
 	}
