@@ -19,34 +19,42 @@ package main
 import (
 	"encoding/json"
 	"fmt"
+	"strconv"
 	"time"
 
-	lib "github.com/coreos/rkt/lib"
-	pkgPod "github.com/coreos/rkt/pkg/pod"
+	lib "github.com/rkt/rkt/lib"
+	pkgPod "github.com/rkt/rkt/pkg/pod"
 	"github.com/spf13/cobra"
+	"golang.org/x/net/context"
 )
 
 var (
 	cmdStatus = &cobra.Command{
-		Use:   "status [--wait] UUID",
+		Use:   "status [--wait=bool|timeout] [--wait-ready=bool|timeout] UUID",
 		Short: "Check the status of a rkt pod",
-		Long: `Prints assorted information about the pod such as its state, pid and exit
-status`,
+		Long: `Prints assorted information about the pod such as its state, pid and exit status.
+
+The --wait and --wait-ready flags accept boolean or timeout values. If set to true, wait indefinitely. If set to false, don't wait at all.
+They can also be set to a duration. If the duration is less than zero, wait indefinitely. If the duration is zero, don't wait at all.`,
 		Run: runWrapper(runStatus),
 	}
-	flagWait bool
+	flagWait      string
+	flagWaitReady string
 )
 
 const (
-	overlayStatusDirTemplate = "overlay/%s/upper/rkt/status"
-	regularStatusDir         = "stage1/rootfs/rkt/status"
-	cmdStatusName            = "status"
+	regularStatusDir = "stage1/rootfs/rkt/status"
+	cmdStatusName    = "status"
 )
 
 func init() {
 	cmdRkt.AddCommand(cmdStatus)
-	cmdStatus.Flags().BoolVar(&flagWait, "wait", false, "toggle waiting for the pod to exit")
-	cmdStatus.Flags().Var(&flagFormat, "format", "choose the output format, allowed format includes 'json', 'json-pretty'. If empty, then the result is printed as key value pairs")
+	cmdStatus.Flags().StringVar(&flagWait, "wait", "false", `toggles waiting for the pod to finish. Use the output to determine the actual terminal state.`)
+	cmdStatus.Flags().StringVar(&flagWaitReady, "wait-ready", "false", `toggles waiting until the pod is ready.`)
+	cmdStatus.Flags().Var(&flagFormat, "format", `choose the output format. Allowed format includes 'json', 'json-pretty'. If empty, then the result is printed as key value pairs`)
+
+	cmdStatus.Flags().Lookup("wait").NoOptDefVal = "true"
+	cmdStatus.Flags().Lookup("wait-ready").NoOptDefVal = "true"
 }
 
 func runStatus(cmd *cobra.Command, args []string) (exit int) {
@@ -55,6 +63,18 @@ func runStatus(cmd *cobra.Command, args []string) (exit int) {
 		return 254
 	}
 
+	dWait, err := parseDuration(flagWait)
+	if err != nil {
+		cmd.Usage()
+		return 254
+	}
+
+	dReady, err := parseDuration(flagWaitReady)
+	if err != nil {
+		cmd.Usage()
+		return 254
+	}
+
 	p, err := pkgPod.PodFromUUIDString(getDataDir(), args[0])
 	if err != nil {
 		stderr.PrintE("problem retrieving pod", err)
@@ -62,9 +82,16 @@ func runStatus(cmd *cobra.Command, args []string) (exit int) {
 	}
 	defer p.Close()
 
-	if flagWait {
-		if err := p.WaitExited(); err != nil {
-			stderr.PrintE("unable to wait for pod", err)
+	if dReady != 0 {
+		if err := p.WaitReady(newContext(dReady)); err != nil {
+			stderr.PrintE("error waiting for pod readiness", err)
+			return 254
+		}
+	}
+
+	if dWait != 0 {
+		if err := p.WaitFinished(newContext(dWait)); err != nil {
+			stderr.PrintE("error waiting for pod to finish", err)
 			return 254
 		}
 	}
@@ -77,6 +104,37 @@ func runStatus(cmd *cobra.Command, args []string) (exit int) {
 	return 0
 }
 
+// parseDuration converts the given string s to a duration value.
+// If it is empty string or a true boolean value according to strconv.ParseBool, a negative duration is returned.
+// If the boolean value is false, a 0 duration is returned.
+// If the string s is a duration value, then it is returned.
+// It returns an error if the duration conversion failed.
+func parseDuration(s string) (time.Duration, error) {
+	if s == "" {
+		return time.Duration(-1), nil
+	}
+
+	b, err := strconv.ParseBool(s)
+
+	switch {
+	case err != nil:
+		return time.ParseDuration(s)
+	case b:
+		return time.Duration(-1), nil
+	}
+
+	return time.Duration(0), nil
+}
+
+// newContext returns a new context with timeout t if t > 0.
+func newContext(t time.Duration) context.Context {
+	ctx := context.Background()
+	if t > 0 {
+		ctx, _ = context.WithTimeout(ctx, t)
+	}
+	return ctx
+}
+
 // getExitStatuses returns a map of the statuses of the pod.
 func getExitStatuses(p *pkgPod.Pod) (map[string]int, error) {
 	_, manifest, err := p.PodManifest()
@@ -144,38 +202,24 @@ func printStatus(p *pkgPod.Pod) error {
 		stdout.Printf("networks=%s", fmtNets(p.Nets))
 	}
 
-	if state == pkgPod.Running || state == pkgPod.Deleting || state == pkgPod.ExitedDeleting || state == pkgPod.Exited || state == pkgPod.ExitedGarbage {
-		var pid int
-		pidCh := make(chan int, 1)
+	if !(state == pkgPod.Running || state == pkgPod.Deleting || state == pkgPod.ExitedDeleting || state == pkgPod.Exited || state == pkgPod.ExitedGarbage) {
+		return nil
+	}
 
-		// Wait slightly because the pid file might not be written yet when the state changes to 'Running'.
-		go func() {
-			for {
-				pid, err := p.Pid()
-				if err == nil {
-					pidCh <- pid
-					return
-				}
-				time.Sleep(time.Millisecond * 100)
-			}
-		}()
+	if pid, err := p.Pid(); err == nil {
+		// the pid file might not be written yet when the state changes to 'Running'
+		// it may also never be written if systemd never executes (e.g.: a bad command)
+		stdout.Printf("pid=%d", pid)
+	}
+	stdout.Printf("exited=%t", (state == pkgPod.Exited || state == pkgPod.ExitedGarbage))
 
-		select {
-		case pid = <-pidCh:
-		case <-time.After(time.Second):
-			return fmt.Errorf("unable to get PID for pod %q: %v", p.UUID, err)
+	if state != pkgPod.Running {
+		stats, err := getExitStatuses(p)
+		if err != nil {
+			return fmt.Errorf("unable to get exit statuses for pod %q: %v", p.UUID, err)
 		}
-
-		stdout.Printf("pid=%d\nexited=%t", pid, (state == pkgPod.Exited || state == pkgPod.ExitedGarbage))
-
-		if state != pkgPod.Running {
-			stats, err := getExitStatuses(p)
-			if err != nil {
-				return fmt.Errorf("unable to get exit statuses for pod %q: %v", p.UUID, err)
-			}
-			for app, stat := range stats {
-				stdout.Printf("app-%s=%d", app, stat)
-			}
+		for app, stat := range stats {
+			stdout.Printf("app-%s=%d", app, stat)
 		}
 	}
 	return nil
