@@ -10,13 +10,13 @@ if ! [[ "$0" =~ "scripts/genproto.sh" ]]; then
 	exit 255
 fi
 
-if ! [[ $(protoc --version) =~ "3.0.0" ]]; then
-	echo "could not find protoc 3.0.0, is it installed + in PATH?"
+if ! [[ $(protoc --version) =~ "3.1.0" ]]; then
+	echo "could not find protoc 3.1.0, is it installed + in PATH?"
 	exit 255
 fi
 
 export PATH=.:${PATH}
-cd $GOPATH/src/github.com/coreos/rkt
+cd $GOPATH/src/github.com/rkt/rkt
 
 echo "building protoc-gen-go"
 pushd vendor/github.com/golang/protobuf/protoc-gen-go
