@@ -40,20 +40,20 @@ import (
 	"github.com/appc/spec/schema"
 	"github.com/appc/spec/schema/types"
 	cnitypes "github.com/containernetworking/cni/pkg/types"
-	"github.com/coreos/rkt/common"
-	"github.com/coreos/rkt/common/apps"
-	commonnet "github.com/coreos/rkt/common/networking"
-	"github.com/coreos/rkt/common/overlay"
-	"github.com/coreos/rkt/pkg/aci"
-	"github.com/coreos/rkt/pkg/fileutil"
-	"github.com/coreos/rkt/pkg/label"
-	"github.com/coreos/rkt/pkg/sys"
-	"github.com/coreos/rkt/pkg/tpm"
-	"github.com/coreos/rkt/pkg/user"
-	"github.com/coreos/rkt/store/imagestore"
-	"github.com/coreos/rkt/store/treestore"
-	"github.com/coreos/rkt/version"
 	"github.com/hashicorp/errwrap"
+	"github.com/rkt/rkt/common"
+	"github.com/rkt/rkt/common/apps"
+	commonnet "github.com/rkt/rkt/common/networking"
+	"github.com/rkt/rkt/common/overlay"
+	"github.com/rkt/rkt/pkg/aci"
+	"github.com/rkt/rkt/pkg/fileutil"
+	"github.com/rkt/rkt/pkg/label"
+	"github.com/rkt/rkt/pkg/sys"
+	"github.com/rkt/rkt/pkg/tpm"
+	"github.com/rkt/rkt/pkg/user"
+	"github.com/rkt/rkt/store/imagestore"
+	"github.com/rkt/rkt/store/treestore"
+	"github.com/rkt/rkt/version"
 )
 
 var debugEnabled bool
@@ -61,17 +61,16 @@ var debugEnabled bool
 // PrepareConfig defines the configuration parameters required by Prepare
 type PrepareConfig struct {
 	*CommonConfig
-	Apps               *apps.Apps            // apps to prepare
-	InheritEnv         bool                  // inherit parent environment into apps
-	ExplicitEnv        []string              // always set these environment variables for all the apps
-	EnvFromFile        []string              // environment variables loaded from files, set for all the apps
-	Ports              []types.ExposedPort   // list of ports that rkt will expose on the host
-	UseOverlay         bool                  // prepare pod with overlay fs
-	SkipTreeStoreCheck bool                  // skip checking the treestore before rendering
-	PodManifest        string                // use the pod manifest specified by the user, this will ignore flags such as '--volume', '--port', etc.
-	PrivateUsers       *user.UidRange        // user namespaces
-	UserAnnotations    types.UserAnnotations // user annotations for the pod.
-	UserLabels         types.UserLabels      // user labels for the pod.
+	Apps            *apps.Apps            // apps to prepare
+	InheritEnv      bool                  // inherit parent environment into apps
+	ExplicitEnv     []string              // always set these environment variables for all the apps
+	EnvFromFile     []string              // environment variables loaded from files, set for all the apps
+	Ports           []types.ExposedPort   // list of ports that rkt will expose on the host
+	UseOverlay      bool                  // prepare pod with overlay fs
+	PodManifest     string                // use the pod manifest specified by the user, this will ignore flags such as '--volume', '--port', etc.
+	PrivateUsers    *user.UidRange        // user namespaces
+	UserAnnotations types.UserAnnotations // user annotations for the pod.
+	UserLabels      types.UserLabels      // user labels for the pod.
 }
 
 // RunConfig defines the configuration parameters needed by Run
@@ -92,21 +91,23 @@ type RunConfig struct {
 	InsecureSeccomp      bool           // Do not add seccomp restrictions
 	UseOverlay           bool           // run pod with overlay fs
 	HostsEntries         HostsEntries   // The entries in /etc/hosts
+	IPCMode              string         // whether to stay in the host IPC namespace
 }
 
 // CommonConfig defines the configuration shared by both Run and Prepare
 type CommonConfig struct {
-	DataDir      string            // The path to the data directory, e.g. /var/lib/rkt/pods
-	Store        *imagestore.Store // store containing all of the configured application images
-	TreeStore    *treestore.Store  // store containing all of the configured application images
-	Stage1Image  types.Hash        // stage1 image containing usable /init and /enter entrypoints
-	UUID         *types.UUID       // UUID of the pod
-	RootHash     string            // Hash of the root filesystem
-	ManifestData string            // The pod manifest data
-	Debug        bool
-	MountLabel   string // selinux label to use for fs
-	ProcessLabel string // selinux label to use
-	Mutable      bool   // whether this pod is mutable
+	DataDir      string                        // The path to the data directory, e.g. /var/lib/rkt/pods
+	Store        *imagestore.Store             // store containing all of the configured application images
+	TreeStore    *treestore.Store              // store containing all of the configured application images
+	Stage1Image  types.Hash                    // stage1 image containing usable /init and /enter entrypoints
+	UUID         *types.UUID                   // UUID of the pod
+	RootHash     string                        // hash of the root filesystem
+	ManifestData string                        // the pod manifest data
+	Debug        bool                          // debug mode
+	MountLabel   string                        // SELinux label to use for fs
+	ProcessLabel string                        // SELinux label to use
+	Mutable      bool                          // whether this pod is mutable
+	Annotations  map[types.ACIdentifier]string // pod-level annotations, for internal/experimental usage
 }
 
 // HostsEntries encapsulates the entries in an etc-hosts file: mapping from IP
@@ -155,27 +156,16 @@ func mergeEnvs(appEnv *types.Environment, env []string, override bool) {
 	}
 }
 
-func imageNameToAppName(name types.ACIdentifier) (*types.ACName, error) {
-	parts := strings.Split(name.String(), "/")
-	last := parts[len(parts)-1]
-
-	sn, err := types.SanitizeACName(last)
-	if err != nil {
-		return nil, err
-	}
-
-	return types.MustACName(sn), nil
-}
-
 // deduplicateMPs removes Mounts with duplicated paths. If there's more than
 // one Mount with the same path, it keeps the first one encountered.
 func deduplicateMPs(mounts []schema.Mount) []schema.Mount {
 	var res []schema.Mount
 	seen := make(map[string]struct{})
 	for _, m := range mounts {
-		if _, ok := seen[m.Path]; !ok {
+		cleanPath := path.Clean(m.Path)
+		if _, ok := seen[cleanPath]; !ok {
 			res = append(res, m)
-			seen[m.Path] = struct{}{}
+			seen[cleanPath] = struct{}{}
 		}
 	}
 	return res
@@ -210,84 +200,34 @@ func generatePodManifest(cfg PrepareConfig, dir string) ([]byte, error) {
 			return errwrap.Wrap(errors.New("error getting the manifest"), err)
 		}
 
-		var appName *types.ACName
-		if app.Name != "" {
-			appName, err = types.NewACName(app.Name)
-			if err != nil {
-				return errwrap.Wrap(errors.New("invalid app name format"), err)
-			}
-		} else {
-			appName, err = imageNameToAppName(am.Name)
+		if app.Name == "" {
+			appName, err := common.ImageNameToAppName(am.Name)
 			if err != nil {
 				return errwrap.Wrap(errors.New("error converting image name to app name"), err)
 			}
+			app.Name = appName.String()
+		}
+
+		appName, err := types.NewACName(app.Name)
+		if err != nil {
+			return errwrap.Wrap(errors.New("invalid app name format"), err)
 		}
 
 		if _, err := prepareAppImage(cfg, *appName, img, dir, cfg.UseOverlay); err != nil {
 			return errwrap.Wrap(fmt.Errorf("error preparing image %s", img), err)
 		}
 		if pm.Apps.Get(*appName) != nil {
-			return fmt.Errorf("error: multiple apps with name %s", am.Name)
+			return fmt.Errorf("error: multiple apps with name %s", app.Name)
 		}
 		if am.App == nil && app.Exec == "" {
 			return fmt.Errorf("error: image %s has no app section and --exec argument is not provided", img)
 		}
-		ra := schema.RuntimeApp{
-			// TODO(vc): leverage RuntimeApp.Name for disambiguating the apps
-			Name: *appName,
-			App:  am.App,
-			Image: schema.RuntimeImage{
-				Name:   &am.Name,
-				ID:     img,
-				Labels: am.Labels,
-			},
-			Mounts:         MergeMounts(cfg.Apps.Mounts, app.Mounts),
-			ReadOnlyRootFS: app.ReadOnlyRootFS,
-		}
 
-		if app.Exec != "" {
-			// Create a minimal App section if not present
-			if am.App == nil {
-				ra.App = &types.App{
-					User:  strconv.Itoa(os.Getuid()),
-					Group: strconv.Itoa(os.Getgid()),
-				}
-			}
-			ra.App.Exec = []string{app.Exec}
-		}
-
-		if app.Args != nil {
-			ra.App.Exec = append(ra.App.Exec, app.Args...)
-		}
-
-		if app.WorkingDir != "" {
-			ra.App.WorkingDirectory = app.WorkingDir
-		}
-
-		if err := prepareIsolators(app, ra.App); err != nil {
+		ra, err := generateRuntimeApp(app, am, cfg.Apps.Mounts)
+		if err != nil {
 			return err
 		}
 
-		if app.User != "" {
-			ra.App.User = app.User
-		}
-
-		if app.Group != "" {
-			ra.App.Group = app.Group
-		}
-
-		if app.SupplementaryGIDs != nil {
-			ra.App.SupplementaryGIDs = app.SupplementaryGIDs
-		}
-
-		if app.UserAnnotations != nil {
-			ra.App.UserAnnotations = app.UserAnnotations
-		}
-
-		if app.UserLabels != nil {
-			ra.App.UserLabels = app.UserLabels
-		}
-
 		// loading the environment from the lowest priority to highest
 		if cfg.InheritEnv {
 			// Inherit environment does not override app image environment
@@ -297,15 +237,8 @@ func generatePodManifest(cfg PrepareConfig, dir string) ([]byte, error) {
 		mergeEnvs(&ra.App.Environment, cfg.EnvFromFile, true)
 		mergeEnvs(&ra.App.Environment, cfg.ExplicitEnv, true)
 
-		if app.Environments != nil {
-			envs := make([]string, 0, len(app.Environments))
-			for name, value := range app.Environments {
-				envs = append(envs, fmt.Sprintf("%s=%s", name, value))
-			}
-			mergeEnvs(&ra.App.Environment, envs, true)
-		}
-
 		pm.Apps = append(pm.Apps, ra)
+
 		return nil
 	}); err != nil {
 		return nil, err
@@ -329,6 +262,14 @@ func generatePodManifest(cfg PrepareConfig, dir string) ([]byte, error) {
 	pm.UserAnnotations = cfg.UserAnnotations
 	pm.UserLabels = cfg.UserLabels
 
+	// Add internal annotations for rkt experiments
+	for k, v := range cfg.Annotations {
+		if _, ok := pm.Annotations.Get(k.String()); ok {
+			continue
+		}
+		pm.Annotations.Set(k, v)
+	}
+
 	pmb, err := json.Marshal(pm)
 	if err != nil {
 		return nil, errwrap.Wrap(errors.New("error marshalling pod manifest"), err)
@@ -725,6 +666,14 @@ func Run(cfg RunConfig, dir string, dataDir string) {
 		args = append(args, "--mutable")
 	}
 
+	if cfg.IPCMode != "" {
+		if interfaceVersionSupportsIPCMode(s1v) {
+			args = append(args, "--ipc="+cfg.IPCMode)
+		} else {
+			log.Printf("warning: --ipc option is not supported by stage1")
+		}
+	}
+
 	args = append(args, cfg.UUID.String())
 
 	// make sure the lock fd stays open across exec
@@ -785,19 +734,6 @@ func prepareAppImage(cfg PrepareConfig, appName types.ACName, img types.Hash, cd
 			return "", errwrap.Wrap(errors.New("error rendering tree image"), err)
 		}
 
-		if !cfg.SkipTreeStoreCheck {
-			hash, err := cfg.TreeStore.Check(treeStoreID)
-			if err != nil {
-				log.PrintE("warning: tree cache is in a bad state.  Rebuilding...", err)
-				var err error
-				treeStoreID, hash, err = cfg.TreeStore.Render(img.String(), true)
-				if err != nil {
-					return "", errwrap.Wrap(errors.New("error rendering tree image"), err)
-				}
-			}
-			cfg.CommonConfig.RootHash = hash
-		}
-
 		if err := ioutil.WriteFile(common.AppTreeStoreIDPath(cdir, appName), []byte(treeStoreID), common.DefaultRegularFilePerm); err != nil {
 			return "", errwrap.Wrap(errors.New("error writing app treeStoreID"), err)
 		}
@@ -902,19 +838,6 @@ func prepareStage1Image(cfg PrepareConfig, cdir string) error {
 		return errwrap.Wrap(errors.New("error rendering tree image"), err)
 	}
 
-	if !cfg.SkipTreeStoreCheck {
-		hash, err := cfg.TreeStore.Check(treeStoreID)
-		if err != nil {
-			log.Printf("warning: tree cache is in a bad state: %v. Rebuilding...", err)
-			var err error
-			treeStoreID, hash, err = cfg.TreeStore.Render(cfg.Stage1Image.String(), true)
-			if err != nil {
-				return errwrap.Wrap(errors.New("error rendering tree image"), err)
-			}
-		}
-		cfg.CommonConfig.RootHash = hash
-	}
-
 	if err := writeManifest(*cfg.CommonConfig, cfg.Stage1Image, s1); err != nil {
 		return errwrap.Wrap(errors.New("error writing manifest"), err)
 	}
@@ -944,8 +867,7 @@ func setupStage1Image(cfg RunConfig, cdir string, useOverlay bool) error {
 			return err
 		}
 
-		// pass an empty appName: make sure it remains consistent with
-		// overlayStatusDirTemplate
+		// pass an empty appName
 		if err := overlayRender(cfg, string(treeStoreID), cdir, s1, ""); err != nil {
 			return errwrap.Wrap(errors.New("error rendering overlay filesystem"), err)
 		}
