@@ -40,23 +40,24 @@ import (
 	"github.com/godbus/dbus/introspect"
 	"github.com/hashicorp/errwrap"
 
-	stage1common "github.com/coreos/rkt/stage1/common"
-	stage1commontypes "github.com/coreos/rkt/stage1/common/types"
-	stage1initcommon "github.com/coreos/rkt/stage1/init/common"
+	stage1common "github.com/rkt/rkt/stage1/common"
+	stage1commontypes "github.com/rkt/rkt/stage1/common/types"
+	stage1initcommon "github.com/rkt/rkt/stage1/init/common"
 
-	"github.com/coreos/rkt/common"
-	"github.com/coreos/rkt/common/cgroup"
-	"github.com/coreos/rkt/common/cgroup/v1"
-	"github.com/coreos/rkt/common/cgroup/v2"
-	commonnet "github.com/coreos/rkt/common/networking"
-	"github.com/coreos/rkt/networking"
-	pkgflag "github.com/coreos/rkt/pkg/flag"
-	"github.com/coreos/rkt/pkg/fs"
-	rktlog "github.com/coreos/rkt/pkg/log"
-	"github.com/coreos/rkt/pkg/sys"
-	"github.com/coreos/rkt/stage1/init/kvm"
-	"github.com/coreos/rkt/stage1/init/kvm/hypervisor/hvlkvm"
-	"github.com/coreos/rkt/stage1/init/kvm/hypervisor/hvqemu"
+	"github.com/rkt/rkt/common"
+	"github.com/rkt/rkt/common/cgroup"
+	"github.com/rkt/rkt/common/cgroup/v1"
+	"github.com/rkt/rkt/common/cgroup/v2"
+	commonnet "github.com/rkt/rkt/common/networking"
+	"github.com/rkt/rkt/networking"
+	pkgflag "github.com/rkt/rkt/pkg/flag"
+	"github.com/rkt/rkt/pkg/fs"
+	rktlog "github.com/rkt/rkt/pkg/log"
+	"github.com/rkt/rkt/pkg/sys"
+	"github.com/rkt/rkt/pkg/user"
+	"github.com/rkt/rkt/stage1/init/kvm"
+	"github.com/rkt/rkt/stage1/init/kvm/hypervisor/hvlkvm"
+	"github.com/rkt/rkt/stage1/init/kvm/hypervisor/hvqemu"
 )
 
 const (
@@ -102,7 +103,6 @@ func mirrorLocalZoneInfo(root string) {
 
 var (
 	debug       bool
-	interactive bool
 	localhostIP net.IP
 	localConfig string
 	log         *rktlog.Logger
@@ -114,10 +114,10 @@ func parseFlags() *stage1commontypes.RuntimePod {
 	rp := stage1commontypes.RuntimePod{}
 
 	flag.BoolVar(&debug, "debug", false, "Run in debug mode")
-	flag.BoolVar(&interactive, "interactive", false, "The pod is interactive")
 	flag.StringVar(&localConfig, "local-config", common.DefaultLocalConfigDir, "Local config path")
 
 	// These flags are persisted in the PodRuntime
+	flag.BoolVar(&rp.Interactive, "interactive", false, "The pod is interactive")
 	flag.BoolVar(&rp.Mutable, "mutable", false, "Enable mutable operations on this pod, including starting an empty one")
 	flag.Var(&rp.NetList, "net", "Setup networking")
 	flag.StringVar(&rp.PrivateUsers, "private-users", "", "Run within user namespace. Can be set to [=UIDBASE[:NUIDS]]")
@@ -134,6 +134,7 @@ func parseFlags() *stage1commontypes.RuntimePod {
 		"hosts":  "default",
 	})
 	flag.Var(dnsConfMode, "dns-conf-mode", "DNS config file modes")
+	flag.StringVar(&rp.IPCMode, "ipc", "", "IPC mode --ipc=[auto|private|parent]")
 
 	flag.Parse()
 
@@ -190,7 +191,7 @@ func machinedRegister() bool {
 	return found == 2
 }
 
-func installAssets() error {
+func installAssets(systemdVersion int) error {
 	systemctlBin, err := common.LookupPath("systemctl", os.Getenv("PATH"))
 	if err != nil {
 		return err
@@ -250,12 +251,19 @@ func installAssets() error {
 		proj2aci.GetAssetString(systemdShutdownBin, systemdShutdownBin),
 	}
 
+	// systemd-journal-flush.service was added in systemd-v233. Required to place
+	// the logs in /var/log/journal instead of /run/log/journal. See:
+	// https://github.com/systemd/systemd/commit/f78273c8dacf678cc8fd7387f678e6344a99405c
+	if systemdVersion >= 233 {
+		assets = append(assets, proj2aci.GetAssetString(fmt.Sprintf("%s/systemd-journal-flush.service", systemdUnitsPath), fmt.Sprintf("%s/systemd-journald.service", systemdUnitsPath)))
+	}
+
 	return proj2aci.PrepareAssets(assets, "./stage1/rootfs/", nil)
 }
 
 // getArgsEnv returns the nspawn or lkvm args and env according to the flavor
 // as the first two return values respectively.
-func getArgsEnv(p *stage1commontypes.Pod, flavor string, canMachinedRegister bool, debug bool, n *networking.Networking) ([]string, []string, error) {
+func getArgsEnv(p *stage1commontypes.Pod, flavor string, canMachinedRegister bool, debug bool, n *networking.Networking, parentIPC bool) ([]string, []string, error) {
 	var args []string
 	env := os.Environ()
 
@@ -264,18 +272,6 @@ func getArgsEnv(p *stage1commontypes.Pod, flavor string, canMachinedRegister boo
 		return nil, nil, errwrap.Wrap(errors.New("failed to create flavor symlink"), err)
 	}
 
-	// set hostname inside pod
-	// According to systemd manual (https://www.freedesktop.org/software/systemd/man/hostname.html) :
-	// "The /etc/hostname file configures the name of the local system that is set
-	// during boot using the sethostname system call"
-	if p.Hostname == "" {
-		p.Hostname = stage1initcommon.GetMachineID(p)
-	}
-	hostnamePath := filepath.Join(common.Stage1RootfsPath(p.Root), "etc/hostname")
-	if err := ioutil.WriteFile(hostnamePath, []byte(p.Hostname), 0644); err != nil {
-		return nil, nil, fmt.Errorf("error writing %s, %s", hostnamePath, err)
-	}
-
 	// systemd-nspawn needs /etc/machine-id to link the container's journal
 	// to the host. Since systemd-v230, /etc/machine-id is mandatory, see
 	// https://github.com/systemd/systemd/commit/e01ff70a77e781734e1e73a2238af2e9bf7967a8
@@ -291,7 +287,7 @@ func getArgsEnv(p *stage1commontypes.Pod, flavor string, canMachinedRegister boo
 		// kernel and hypervisor binaries are located relative to the working directory
 		// of init (/var/lib/rkt/..../uuid)
 		// TODO: move to path.go
-		kernelPath := filepath.Join(common.Stage1RootfsPath(p.Root), "bzImage")
+		kernelPath := filepath.Join(common.Stage1RootfsPath(p.Root), "kernel_image")
 		netDescriptions := kvm.GetNetworkDescriptions(n)
 
 		cpu, mem := kvm.GetAppsResources(p.Manifest.Apps)
@@ -330,7 +326,7 @@ func getArgsEnv(p *stage1commontypes.Pod, flavor string, canMachinedRegister boo
 		args = append(args, hvStartCmd...)
 
 		// lkvm requires $HOME to be defined,
-		// see https://github.com/coreos/rkt/issues/1393
+		// see https://github.com/rkt/rkt/issues/1393
 		if os.Getenv("HOME") == "" {
 			env = append(env, "HOME=/root")
 		}
@@ -423,7 +419,7 @@ func getArgsEnv(p *stage1commontypes.Pod, flavor string, canMachinedRegister boo
 		}
 
 		// Copy systemd, bash, etc. in stage1 at run-time
-		if err := installAssets(); err != nil {
+		if err := installAssets(version); err != nil {
 			return nil, nil, errwrap.Wrap(errors.New("cannot install assets from the host"), err)
 		}
 
@@ -449,7 +445,10 @@ func getArgsEnv(p *stage1commontypes.Pod, flavor string, canMachinedRegister boo
 
 	machineIDBytes := append([]byte(machineID), '\n')
 	if err := ioutil.WriteFile(mPath, machineIDBytes, 0644); err != nil {
-		log.FatalE("error writing /etc/machine-id", err)
+		return nil, nil, errwrap.Wrap(errors.New("error writing /etc/machine-id"), err)
+	}
+	if err := user.ShiftFiles([]string{mPath}, &p.UidRange); err != nil {
+		return nil, nil, errwrap.Wrap(errors.New("error shifting /etc/machine-id"), err)
 	}
 
 	// link journal only if the host is running systemd
@@ -477,6 +476,10 @@ func getArgsEnv(p *stage1commontypes.Pod, flavor string, canMachinedRegister boo
 		env = append(env, "SYSTEMD_LOG_LEVEL=err") // silence log_warning too
 	}
 
+	if parentIPC {
+		env = append(env, "SYSTEMD_NSPAWN_SHARE_NS_IPC=true")
+	}
+
 	env = append(env, "SYSTEMD_NSPAWN_CONTAINER_SERVICE=rkt")
 	// TODO (alepuccetti) remove this line when rkt will use cgroup namespace
 	// If the kernel has the cgroup namespace enabled, systemd v232 will use it by default.
@@ -588,12 +591,45 @@ func stage1(rp *stage1commontypes.RuntimePod) int {
 		diag.Printf,
 	)
 
+	// set hostname inside pod
+	// According to systemd manual (https://www.freedesktop.org/software/systemd/man/hostname.html) :
+	// "The /etc/hostname file configures the name of the local system that is set
+	// during boot using the sethostname system call"
+	if p.Hostname == "" {
+		p.Hostname = stage1initcommon.GetMachineID(p)
+	}
+	hostnamePath := filepath.Join(common.Stage1RootfsPath(p.Root), "etc/hostname")
+	if err := ioutil.WriteFile(hostnamePath, []byte(p.Hostname), 0644); err != nil {
+		log.PrintE("error writing "+hostnamePath, err)
+		return 254
+	}
+	if err := user.ShiftFiles([]string{hostnamePath}, &p.UidRange); err != nil {
+		log.PrintE("error shifting "+hostnamePath, err)
+	}
+
 	if p.ResolvConfMode == "host" {
 		stage1initcommon.UseHostResolv(mnt, root)
 	}
 
-	if p.EtcHostsMode == "host" {
+	// Set up the hosts file.
+	// We write <stage1>/etc/rkt-hosts if we want to override each app's hosts,
+	// and <stage1>/etc/hosts-fallback if we want to let the app "win"
+	// Either way, we should add our hostname to it, unless the hosts's
+	// /etc/hosts is bind-mounted in.
+	if p.EtcHostsMode == "host" { // We should bind-mount the hosts's /etc/hosts
 		stage1initcommon.UseHostHosts(mnt, root)
+	} else if p.EtcHostsMode == "default" { // Create hosts-fallback
+		hostsFile := filepath.Join(common.Stage1RootfsPath(p.Root), "etc", "hosts-fallback")
+		if err := stage1initcommon.AddHostsEntry(hostsFile, "127.0.0.1", p.Hostname); err != nil {
+			log.PrintE("Failed to write hostname to "+hostsFile, err)
+			return 254
+		}
+	} else if p.EtcHostsMode == "stage0" { // The stage0 has created rkt-hosts
+		hostsFile := filepath.Join(common.Stage1RootfsPath(p.Root), "etc", "rkt-hosts")
+		if err := stage1initcommon.AddHostsEntry(hostsFile, "127.0.0.1", p.Hostname); err != nil {
+			log.PrintE("Failed to write hostname to "+hostsFile, err)
+			return 254
+		}
 	}
 
 	if p.Mutable {
@@ -601,7 +637,7 @@ func stage1(rp *stage1commontypes.RuntimePod) int {
 			log.FatalE("cannot initialize mutable environment", err)
 		}
 	} else {
-		if err = stage1initcommon.ImmutableEnv(p, interactive); err != nil {
+		if err = stage1initcommon.ImmutableEnv(p); err != nil {
 			log.FatalE("cannot initialize immutable environment", err)
 		}
 	}
@@ -624,7 +660,26 @@ func stage1(rp *stage1commontypes.RuntimePod) int {
 	}
 	diag.Printf("canMachinedRegister %t", canMachinedRegister)
 
-	args, env, err := getArgsEnv(p, flavor, canMachinedRegister, debug, n)
+	// --ipc=[auto|private|parent]
+	// default to private
+	parentIPC := false
+	switch p.IPCMode {
+	case "parent":
+		parentIPC = true
+	case "private":
+		parentIPC = false
+	case "auto":
+		fallthrough
+	case "":
+		parentIPC = false
+	default:
+		log.Fatalf("unknown value for --ipc parameter: %v", p.IPCMode)
+	}
+	if parentIPC && flavor == "kvm" {
+		log.Fatal("flavor kvm requires private IPC namespace (try to remove --ipc)")
+	}
+
+	args, env, err := getArgsEnv(p, flavor, canMachinedRegister, debug, n, parentIPC)
 	if err != nil {
 		log.FatalE("cannot get environment", err)
 	}
@@ -729,7 +784,7 @@ func stage1(rp *stage1commontypes.RuntimePod) int {
 func areHostV1CgroupsMounted(enabledV1Cgroups map[int][]string) bool {
 	controllers := v1.GetControllerDirs(enabledV1Cgroups)
 	for _, c := range controllers {
-		if !v1.IsControllerMounted(c) {
+		if mounted, _ := v1.IsControllerMounted(c); !mounted {
 			return false
 		}
 	}
@@ -751,7 +806,11 @@ func mountHostV1Cgroups(m fs.Mounter, enabledCgroups map[int][]string) error {
 		}
 	}
 
-	if !v1.IsControllerMounted("systemd") {
+	mounted, err := v1.IsControllerMounted("systemd")
+	if err != nil {
+		return err
+	}
+	if !mounted {
 		if err := os.MkdirAll(systemdControllerPath, 0700); err != nil {
 			return err
 		}
@@ -773,7 +832,7 @@ func mountContainerV1Cgroups(m fs.Mounter, p *stage1commontypes.Pod, enabledCgro
 		return errwrap.Wrap(errors.New("error creating container cgroups"), err)
 	}
 
-	if err := v1.RemountCgroups(m, stage1Root, enabledCgroups, subcgroup, serviceNames, p.InsecureOptions.DisablePaths); err != nil {
+	if err := v1.RemountCgroups(m, stage1Root, enabledCgroups, subcgroup, p.InsecureOptions.DisablePaths); err != nil {
 		return errwrap.Wrap(errors.New("error restricting container cgroups"), err)
 	}
 
@@ -832,9 +891,14 @@ func getContainerSubCgroup(machineID string, canMachinedRegister, unified bool)
 
 	// when registration is disabled the container will be directly
 	// under the current cgroup so we can look it up in /proc/self/cgroup
+	// Try the systemd slice first, falling back to cpu if that fails (e.g. on
+	// systems not running systemd). See issue #3502.
 	ownV1CgroupPath, err := v1.GetOwnCgroupPath("name=systemd")
 	if err != nil {
-		return "", errwrap.Wrap(errors.New("could not get own v1 cgroup path"), err)
+		ownV1CgroupPath, err = v1.GetOwnCgroupPath("cpu")
+		if err != nil {
+			return "", errwrap.Wrap(errors.New("could not get own v1 cgroup path"), err)
+		}
 	}
 
 	// systemd-nspawn won't work if we are in the root cgroup. In addition,
