@@ -30,10 +30,13 @@ import (
 	"testing"
 	"time"
 
+	"github.com/rkt/rkt/common"
+	"github.com/rkt/rkt/pkg/aci/acitest"
+	"github.com/rkt/rkt/tests/testutils"
+	taas "github.com/rkt/rkt/tests/testutils/aci-server"
+
+	"github.com/appc/spec/schema"
 	"github.com/appc/spec/schema/types"
-	"github.com/coreos/rkt/common"
-	"github.com/coreos/rkt/tests/testutils"
-	taas "github.com/coreos/rkt/tests/testutils/aci-server"
 )
 
 // TestFetchFromFile tests that 'rkt fetch/run/prepare' for a file will always
@@ -93,32 +96,46 @@ func TestFetchAny(t *testing.T) {
 
 	defer os.Remove(imagePath)
 
+	aci_os, aci_arch := common.GetOSArch()
+	os_arch := aci_os + "-" + aci_arch
+
+	etcdVersion := "v3.2.2"
+	etcdImage := "coreos.com/etcd:" + etcdVersion
+	etcdURL := "https://github.com/coreos/etcd/releases/download/" + etcdVersion + "/etcd-" + etcdVersion + "-" + os_arch + ".aci"
+
+	dockerVersion := "latest"
+	dockerImage := "docker://busybox"
+	if aci_arch != "amd64" {
+		dockerImage = "docker://" + aci_arch + "/busybox"
+	}
+	dockerImageV := dockerImage + ":" + dockerVersion
+
 	tests := []struct {
 		args      string
 		image     string
 		imageArgs string
 		finalURL  string
 	}{
-		{"--insecure-options=image --debug fetch", "coreos.com/etcd:v2.1.2", "", "https://github.com/coreos/etcd/releases/download/v2.1.2/etcd-v2.1.2-linux-amd64.aci"},
-		{"--insecure-options=image --debug fetch", "https://github.com/coreos/etcd/releases/download/v2.1.2/etcd-v2.1.2-linux-amd64.aci", "", "https://github.com/coreos/etcd/releases/download/v2.1.2/etcd-v2.1.2-linux-amd64.aci"},
-		{"--insecure-options=image --debug fetch", "docker://busybox", "", "docker://busybox"},
-		{"--insecure-options=image --debug fetch", "docker://busybox:latest", "", "docker://busybox:latest"},
-		{"--insecure-options=image --debug run --mds-register=false", "coreos.com/etcd:v2.1.2", "--exec /etcdctl", "https://github.com/coreos/etcd/releases/download/v2.1.2/etcd-v2.1.2-linux-amd64.aci"},
-		{"--insecure-options=image --debug run --mds-register=false", "https://github.com/coreos/etcd/releases/download/v2.1.2/etcd-v2.1.2-linux-amd64.aci", "--exec /etcdctl", "https://github.com/coreos/etcd/releases/download/v2.1.2/etcd-v2.1.2-linux-amd64.aci"},
-		{"--insecure-options=image --debug run --mds-register=false", "docker://busybox", "", "docker://busybox"},
-		{"--insecure-options=image --debug run --mds-register=false", "docker://busybox:latest", "", "docker://busybox:latest"},
-		{"--insecure-options=image --debug prepare", "https://github.com/coreos/etcd/releases/download/v2.1.2/etcd-v2.1.2-linux-amd64.aci", "", "https://github.com/coreos/etcd/releases/download/v2.1.2/etcd-v2.1.2-linux-amd64.aci"},
-		{"--insecure-options=image --debug prepare", "coreos.com/etcd:v2.1.2", "", "https://github.com/coreos/etcd/releases/download/v2.1.2/etcd-v2.1.2-linux-amd64.aci"},
+		{"--insecure-options=image --debug fetch", etcdImage, "", etcdURL},
+		{"--insecure-options=image --debug fetch", etcdURL, "", etcdURL},
+		{"--insecure-options=image --debug fetch", dockerImage, "", dockerImage},
+		{"--insecure-options=image --debug fetch", dockerImageV, "", dockerImageV},
+		{"--insecure-options=image --debug run --mds-register=false", etcdImage, "--exec /etcdctl", etcdURL},
+		{"--insecure-options=image --debug run --mds-register=false", etcdURL, "--exec /etcdctl", etcdURL},
+		{"--insecure-options=image --debug run --mds-register=false", dockerImage, "", dockerImage},
+		{"--insecure-options=image --debug run --mds-register=false", dockerImageV, "", dockerImageV},
+		{"--insecure-options=image --debug prepare", etcdURL, "", etcdURL},
+		{"--insecure-options=image --debug prepare", etcdImage, "", etcdURL},
 		// test --insecure-options=tls to make sure
-		// https://github.com/coreos/rkt/issues/1829 is not an issue anymore
-		{"--insecure-options=image,tls --debug prepare", "docker://busybox", "", "docker://busybox"},
-		{"--insecure-options=image --debug prepare", "docker://busybox:latest", "", "docker://busybox:latest"},
+		// https://github.com/rkt/rkt/issues/1829 is not an issue anymore
+		{"--insecure-options=image,tls --debug prepare", dockerImage, "", dockerImage},
+		{"--insecure-options=image --debug prepare", dockerImageV, "", dockerImageV},
 	}
 
 	for _, tt := range tests {
-		testFetchDefault(t, tt.args, tt.image, tt.imageArgs, tt.finalURL)
-		testFetchStoreOnly(t, tt.args, tt.image, tt.imageArgs, tt.finalURL)
-		testFetchNoStore(t, tt.args, tt.image, tt.imageArgs, tt.finalURL)
+		testFetchNew(t, tt.args, tt.image, tt.imageArgs, tt.finalURL)
+		testFetchNever(t, tt.args, tt.image, tt.imageArgs, tt.finalURL)
+		testFetchUpdate(t, tt.args, tt.image, tt.imageArgs, tt.finalURL)
 	}
 }
 
@@ -147,7 +164,7 @@ func TestFetchFullHash(t *testing.T) {
 	}
 }
 
-func testFetchDefault(t *testing.T, arg string, image string, imageArgs string, finalURL string) {
+func testFetchNew(t *testing.T, arg string, image string, imageArgs string, finalURL string) {
 	remoteFetchMsgTpl := `remote fetching from URL %q`
 	storeMsgTpl := `using image from local store for .* %s`
 	if finalURL == "" {
@@ -159,7 +176,7 @@ func testFetchDefault(t *testing.T, arg string, image string, imageArgs string,
 	ctx := testutils.NewRktRunCtx()
 	defer ctx.Cleanup()
 
-	cmd := fmt.Sprintf("%s %s %s %s", ctx.Cmd(), arg, image, imageArgs)
+	cmd := fmt.Sprintf("%s --pull-policy=new %s %s %s", ctx.Cmd(), arg, image, imageArgs)
 
 	// 1. Run cmd with the image not available in the store, should get $remoteFetchMsg.
 	err := runRktAndCheckRegexOutput(t, cmd, remoteFetchMsg)
@@ -173,7 +190,7 @@ func testFetchDefault(t *testing.T, arg string, image string, imageArgs string,
 	runRktAndCheckRegexOutput(t, cmd, storeMsg)
 }
 
-func testFetchStoreOnly(t *testing.T, args string, image string, imageArgs string, finalURL string) {
+func testFetchNever(t *testing.T, args string, image string, imageArgs string, finalURL string) {
 	cannotFetchMsgTpl := `unable to fetch.* image from .* %q`
 	storeMsgTpl := `using image from local store for .* %s`
 	cannotFetchMsg := fmt.Sprintf(cannotFetchMsgTpl, image)
@@ -182,7 +199,7 @@ func testFetchStoreOnly(t *testing.T, args string, image string, imageArgs strin
 	ctx := testutils.NewRktRunCtx()
 	defer ctx.Cleanup()
 
-	cmd := fmt.Sprintf("%s --store-only %s %s %s", ctx.Cmd(), args, image, imageArgs)
+	cmd := fmt.Sprintf("%s --pull-policy=never %s %s %s", ctx.Cmd(), args, image, imageArgs)
 
 	// 1. Run cmd with the image not available in the store should get $cannotFetchMsg.
 	runRktAndCheckRegexOutput(t, cmd, cannotFetchMsg)
@@ -195,7 +212,7 @@ func testFetchStoreOnly(t *testing.T, args string, image string, imageArgs strin
 	runRktAndCheckRegexOutput(t, cmd, storeMsg)
 }
 
-func testFetchNoStore(t *testing.T, args string, image string, imageArgs string, finalURL string) {
+func testFetchUpdate(t *testing.T, args string, image string, imageArgs string, finalURL string) {
 	remoteFetchMsgTpl := `remote fetching from URL %q`
 	remoteFetchMsg := fmt.Sprintf(remoteFetchMsgTpl, finalURL)
 
@@ -206,7 +223,7 @@ func testFetchNoStore(t *testing.T, args string, image string, imageArgs string,
 		t.Skip(fmt.Sprintf("%v, probably a network failure. Skipping...", err))
 	}
 
-	cmd := fmt.Sprintf("%s --no-store %s %s %s", ctx.Cmd(), args, image, imageArgs)
+	cmd := fmt.Sprintf("%s --pull-policy=update %s %s %s", ctx.Cmd(), args, image, imageArgs)
 
 	// 1. Run cmd with the image available in the store, should get $remoteFetchMsg.
 	err := runRktAndCheckRegexOutput(t, cmd, remoteFetchMsg)
@@ -535,11 +552,24 @@ func TestDeferredSignatureDownload(t *testing.T) {
 }
 
 func TestDifferentDiscoveryLabels(t *testing.T) {
-	manifestTemplate := `{"acKind":"ImageManifest","acVersion":"0.8.9","name":"IMG_NAME","labels":[{"name":"version","value":"1.2.0"},{"name":"arch","value":"amd64"},{"name":"os","value":"linux"}]}`
+	const imageName = "localhost/rkt-test-different-discovery-labels-image"
+
+	aci_os, aci_arch := common.GetOSArch()
+
+	manifest, err := acitest.ImageManifestString(&schema.ImageManifest{
+		Name: imageName, Labels: types.Labels{
+			{"version", "1.2.0"},
+			{"arch", aci_arch},
+			{"os", aci_os},
+		},
+	})
+
+	if err != nil {
+		t.Fatalf("unexpected error %v", err)
+	}
+
 	emptyImage := getEmptyImagePath()
-	imageName := "localhost/rkt-test-different-discovery-labels-image"
-	manifest := strings.Replace(manifestTemplate, "IMG_NAME", imageName, -1)
-	tmpDir := createTempDirOrPanic("rkt-TestDifferentDiscoveryLabels-")
+	tmpDir := mustTempDir("rkt-TestDifferentDiscoveryLabels-")
 	defer os.RemoveAll(tmpDir)
 
 	tmpManifest, err := ioutil.TempFile(tmpDir, "manifest")
@@ -551,8 +581,8 @@ func TestDifferentDiscoveryLabels(t *testing.T) {
 	}
 	defer os.Remove(tmpManifest.Name())
 
-	image := patchACI(emptyImage, "rkt-test-different-discovery-labels-image.aci", "--manifest", tmpManifest.Name())
 	imageFileName := fmt.Sprintf("%s.aci", filepath.Base(imageName))
+	image := patchACI(emptyImage, imageFileName, "--manifest", tmpManifest.Name())
 	defer os.Remove(image)
 
 	asc := runSignImage(t, image, 1)
@@ -569,13 +599,18 @@ func TestDifferentDiscoveryLabels(t *testing.T) {
 		t.Fatalf("Failed to populate a file list in test aci server: %v", err)
 	}
 
+	other_arch := "armv7b"
+	if aci_arch == "armv7b" {
+		other_arch = "amd64"
+	}
+
 	tests := []struct {
 		imageName       string
 		expectedMessage string
 	}{
 		{imageName + ":2.0", fmt.Sprintf("requested value for label %q: %q differs from fetched aci label value: %q", "version", "2.0", "1.2.0")},
 		{imageName + ":latest", fmt.Sprintf("requested value for label %q: %q differs from fetched aci label value: %q", "version", "latest", "1.2.0")},
-		{imageName + ",arch=armv7b", fmt.Sprintf("requested value for label %q: %q differs from fetched aci label value: %q", "arch", "armv7b", "amd64")},
+		{imageName + ",arch=" + other_arch, fmt.Sprintf("requested value for label %q: %q differs from fetched aci label value: %q", "arch", other_arch, aci_arch)},
 		{imageName + ",unexistinglabel=bla", fmt.Sprintf("requested label %q not provided by the image manifest", "unexistinglabel")},
 	}
 
