@@ -21,8 +21,8 @@ import (
 	"os"
 	"testing"
 
-	"github.com/coreos/rkt/common"
-	"github.com/coreos/rkt/tests/testutils"
+	"github.com/rkt/rkt/common"
+	"github.com/rkt/rkt/tests/testutils"
 )
 
 // TestNonRootReadInfo tests that non-root users that can do rkt list, rkt image list.
@@ -38,7 +38,9 @@ func TestNonRootReadInfo(t *testing.T) {
 	ctx := testutils.NewRktRunCtx()
 	defer ctx.Cleanup()
 
-	gid, err := common.LookupGid(common.RktGroup)
+	uid, _ := ctx.GetUidGidRktBinOwnerNotRoot()
+
+	rktGid, err := common.LookupGid(common.RktGroup)
 	if err != nil {
 		t.Skipf("Skipping the test because there's no %q group", common.RktGroup)
 	}
@@ -82,7 +84,7 @@ func TestNonRootReadInfo(t *testing.T) {
 
 	imgListCmd := fmt.Sprintf("%s image list", ctx.Cmd())
 	t.Logf("Running %s", imgListCmd)
-	runRktAsGidAndCheckOutput(t, imgListCmd, "inspect-", false, gid)
+	runRktAsUidGidAndCheckOutput(t, imgListCmd, "inspect-", false, false, uid, rktGid)
 }
 
 // TestNonRootFetchRmGCImage tests that non-root users can remove all images but
@@ -91,7 +93,9 @@ func TestNonRootFetchRmGCImage(t *testing.T) {
 	ctx := testutils.NewRktRunCtx()
 	defer ctx.Cleanup()
 
-	gid, err := common.LookupGid(common.RktGroup)
+	uid, _ := ctx.GetUidGidRktBinOwnerNotRoot()
+
+	rktGid, err := common.LookupGid(common.RktGroup)
 	if err != nil {
 		t.Skipf("Skipping the test because there's no %q group", common.RktGroup)
 	}
@@ -117,22 +121,22 @@ func TestNonRootFetchRmGCImage(t *testing.T) {
 	// We can't touch the treestores even as members of the rkt group.
 	imgGCCmd := fmt.Sprintf("%s image gc", ctx.Cmd())
 	t.Logf("Running %s", imgGCCmd)
-	runRktAsGidAndCheckOutput(t, imgGCCmd, "permission denied", true, gid)
+	runRktAsUidGidAndCheckOutput(t, imgGCCmd, "permission denied", false, true, uid, rktGid)
 
 	// Should be able to remove the image fetched by root since we're in the rkt group.
 	imgRmCmd := fmt.Sprintf("%s image rm %s", ctx.Cmd(), rootImgHash)
 	t.Logf("Running %s", imgRmCmd)
-	runRktAsGidAndCheckOutput(t, imgRmCmd, "successfully removed", false, gid)
+	runRktAsUidGidAndCheckOutput(t, imgRmCmd, "successfully removed", false, false, uid, rktGid)
 
 	// Should be able to remove the image fetched by ourselves.
 	nonrootImg := patchTestACI("rkt-inspect-non-root-rm.aci", "--exec=/inspect")
 	defer os.Remove(nonrootImg)
-	nonrootImgHash, err := importImageAndFetchHashAsGid(t, ctx, nonrootImg, "", gid)
+	nonrootImgHash, err := importImageAndFetchHashAsUidGid(t, ctx, nonrootImg, "", uid, rktGid)
 	if err != nil {
 		t.Fatalf("%v", err)
 	}
 
 	imgRmCmd = fmt.Sprintf("%s image rm %s", ctx.Cmd(), nonrootImgHash)
 	t.Logf("Running %s", imgRmCmd)
-	runRktAsGidAndCheckOutput(t, imgRmCmd, "successfully removed", false, gid)
+	runRktAsUidGidAndCheckOutput(t, imgRmCmd, "successfully removed", false, false, uid, rktGid)
 }
