@@ -20,9 +20,9 @@ import (
 	"path/filepath"
 	"strings"
 
-	"github.com/coreos/rkt/tools/common"
-	"github.com/coreos/rkt/tools/common/filelist"
 	"github.com/hashicorp/errwrap"
+	"github.com/rkt/rkt/tools/common"
+	"github.com/rkt/rkt/tools/common/filelist"
 )
 
 const (
